"""Set lib version from `pyproject.toml`.

.. note::

  .. deprecated:: 1.3.6
    Use:

    .. code-block:: python

      __version__ = __import__("importlib.metadata", fromlist=["version"]).version(
          <package name>
      )

    instead

This library allows for setting the version number for a library from
the pyproject.toml file.

Add to your `pyproject.toml` add a new section:

.. code-block:: toml

  [tool.berhoel.helper.set_version]
  version_files = ["berhoel/helper/_version.py"]

Generate the version file:

.. code-block:: shell

  > poetry run set_lib_version
  writing berhoel/helper/_version.py

In the library `__init__.py` just use:

.. code-block:: python

  try:
      from ._version import __version__
  except ImportError:
      __version__ = "0.0.0.invalid0"
"""

from __future__ import annotations

import argparse
from importlib import metadata
from pathlib import Path
import sys
import warnings

import tomli


def build_parser() -> argparse.ArgumentParser:
    """Build cli parser."""
    parser = argparse.ArgumentParser(
        prog="set_lib_version",
        description="Create version files with version number from `pyproject.toml`.",
    )
    parser.add_argument(
        "--version",
        action="version",
        version=f"%(prog)s {metadata.version('bhoelHelper')}",
    )
    return parser


def build() -> None:
    """Create version files with version number from `pyproject.toml`."""
    message = "dont use __version__"
    warnings.warn(message, DeprecationWarning, stacklevel=2)

    parser = build_parser()
    parser.parse_args()

    pyproject = (Path() / "pyproject.toml").resolve()

    with pyproject.open("rb") as conf_inp:
        config = tomli.load(conf_inp)

    version = config["tool"]["poetry"]["version"]
    ver_files = config["tool"]["berhoel"]["helper"]["set_version"]["version_files"]
    for ver_file in ver_files:
        sys.stdout.write(f"writing {ver_file!s}\n")
        with Path(ver_file).resolve().open("w") as target:
            target.write(f'__version__ = "{version}"\n')



"""Misc helper."""

from __future__ import annotations

from contextlib import contextmanager
from datetime import timedelta
from enum import Enum, auto
import sys
import time
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from collections.abc import Callable, Generator


class SwirlSelect(Enum):
    """Selector for swirl style."""

    LINES = auto()
    DOTS = auto()


__swirl_string = {
    SwirlSelect.LINES: r"\|/-",
    SwirlSelect.DOTS: (
        "\N{BRAILLE PATTERN DOTS-123}"  # "⠇"
        "\N{BRAILLE PATTERN DOTS-237}"  # "⡆"
        "\N{BRAILLE PATTERN DOTS-378}"  # "⣄"
        "\N{BRAILLE PATTERN DOTS-678}"  # "⣠"
        "\N{BRAILLE PATTERN DOTS-568}"  # "⢰"
        "\N{BRAILLE PATTERN DOTS-456}"  # "⠸"
        "\N{BRAILLE PATTERN DOTS-145}"  # "⠙"
        "\N{BRAILLE PATTERN DOTS-124}"  # "⠋"
    ),
}


def swirl(style: SwirlSelect = SwirlSelect.LINES) -> Generator:
    r"""Retrun generator to show a swirling life indicator.

    >>> sw = swirl()
    >>> a = [_ for _ in zip(range(5), sw)]
    \...|.../...-...\...
    >>> sw = swirl(SwirlSelect.DOTS)
    >>> a = [_ for _ in zip(range(8), sw)]
    ⠇...⡆...⣄...⣠...⢰...⠸...⠙...⠋...

    Returns
    -------
      `generator`: printing running indicator.
    """
    sw_string = __swirl_string[style]
    while True:
        for c in sw_string:
            sys.stdout.write(f"{c}\r")
            yield


def count_with_msg(msg: str = "loop", start: int = 0) -> Generator:
    """Count variable with start value and message.

    >>> c = count_with_msg("msg", 5)
    >>> print([i for _, i in zip(range(5), c)] == [5, 6, 7, 8, 9])
    msg 1 ...msg 2 ...msg 3 ...msg 4 ...msg 5 ...True
    >>>

    Args:
        msg (str): base message
        start (int): counter start_time

    Returns
    -------
        `generator`: counter with message.
    """
    i = 1
    _count = start
    while True:
        sys.stdout.write(f"{msg} {i} \r")
        yield _count
        _count += 1
        i += 1


@contextmanager
def process_msg_context(msg: str) -> Generator:
    """Provide a context for calling routines and reporting entering and exit.

    >>> with process_msg_context("do something"):
    ...     pass
    do something......do something...done
    >>>

    Args:
        msg (str): message for bracing process.

    Returns
    -------
        `contextmanager`: bracing message.
    """
    sys.stdout.write(f"{msg}...\r")
    yield
    sys.stdout.write(f"{msg}...done\n")


@contextmanager
def timed_process_msg_context(msg: str, time_form: Callable | None = None) -> Generator:
    """Provide a context for calling routines and reporting entering and exit.

    Report spent time.

    >>> with timed_process_msg_context("do something"):
    ...     time.sleep(1)
    do something......do something...done (0:00:01)
    >>> with timed_process_msg_context(
    ...     "do something", lambda t: "{:d}s".format(int(t))
    ... ):
    ...     time.sleep(1)
    do something......do something...done (1s)
    >>>

    Args:
        msg (str): message for bracing process.
        time_form (func): function formatting druntime.

    Returns
    -------
        `contextmanager`: bracing message.
    """
    if time_form is None:

        def time_form(t: float | str) -> timedelta:
            return timedelta(seconds=int(t))

    start_time = time.time()
    sys.stdout.write(f"{msg}...\r")
    yield
    sys.stdout.write(f"{msg}...done ({time_form(time.time() - start_time)})\n")



"""Tests for `berhoel.helper`."""

import time

from berhoel import helper


def test_swirl(capsys):
    """Test swirler."""
    swirl = helper.swirl()
    for expected in r"\|/-\|":
        next(swirl)
        res = capsys.readouterr()
        assert res.out == f"{expected}\r"


def test_swirl_dots(capsys):
    """Test swirler with (braille) dots."""
    swirl = helper.swirl(helper.SwirlSelect.DOTS)
    for expected in "⠇⡆⣄⣠⢰⠸⠙⠋":
        next(swirl)
        res = capsys.readouterr()
        assert res.out == f"{expected}\r"


def test_count_with_message_1(capsys):
    """Test counter with life message."""
    count = helper.count_with_msg()
    for i, j in enumerate(count):
        res = capsys.readouterr()
        assert i == j
        assert res.out == f"loop {i + 1} \r"
        if i > 5:
            break


def test_count_with_message_2(capsys):
    """Test counter with life message."""
    count = helper.count_with_msg(start=10)
    for i, j in enumerate(count):
        res = capsys.readouterr()
        assert i + 10 == j
        assert res.out == f"loop {i + 1} \r"
        if i > 5:
            break


def test_count_with_message_3(capsys):
    """Test counter with life message."""
    count = helper.count_with_msg(msg="msg")
    for i, j in enumerate(count):
        res = capsys.readouterr()
        assert i == j
        assert res.out == f"msg {i + 1} \r"
        if i > 5:
            break


def test_count_with_message_4(capsys):
    """Test counter with life message."""
    count = helper.count_with_msg("alt", 5)
    for i, j in enumerate(count):
        res = capsys.readouterr()
        assert i + 5 == j
        assert res.out == f"alt {i + 1} \r"
        if i > 5:
            break


def test_process_msg_context(capsys):
    """Test process context."""
    with helper.process_msg_context("do something"):
        res = capsys.readouterr()
        assert res.out == "do something...\r"
    res = capsys.readouterr()
    assert res.out == "do something...done\n"


def test_timed_process_msg_context_1(capsys):
    """Test timed process context."""
    with helper.timed_process_msg_context("do something"):
        res = capsys.readouterr()
        time.sleep(1)
        assert res.out == "do something...\r"
    res = capsys.readouterr()
    assert res.out == "do something...done (0:00:01)\n"


def test_timed_process_msg_context_2(capsys):
    """Test timed process context."""
    with helper.timed_process_msg_context("do something", lambda t: f"{int(t):d}s"):
        res = capsys.readouterr()
        time.sleep(1)
        assert res.out == "do something...\r"
    res = capsys.readouterr()
    assert res.out == "do something...done (1s)\n"



"""Testing `berhoel.helper.machar`."""

from berhoel.helper import machar


def test_ibeta():
    assert isinstance(machar.ibeta, int)


def test_it():
    assert isinstance(machar.it, int)


def test_irnd():
    assert isinstance(machar.irnd, int)


def test_ngrd():
    assert isinstance(machar.ngrd, int)


def test_machdep():
    assert isinstance(machar.machdep, int)


def test_negep():
    assert isinstance(machar.negep, int)


def test_iexp():
    assert isinstance(machar.iexp, int)


def test_minexp():
    assert isinstance(machar.minexp, int)


def test_maxexp():
    assert isinstance(machar.maxexp, int)


def test_eps():
    assert isinstance(machar.eps, float)


def test_epsneg():
    assert isinstance(machar.epsneg, float)


def test_xmin():
    assert isinstance(machar.xmin, float)


def test_xmax():
    assert isinstance(machar.xmax, float)
